@@ -12,7 +12,7 @@
  *                         All rights reserved.
  * Copyright (c) 2007      Cisco, Inc.  All rights reserved.
  * Copyright (c) 2007      Voltaire All rights reserved.
- * Copyright (c) 2006-2008 University of Houston.  All rights reserved.
+ * Copyright (c) 2006-2009 University of Houston.  All rights reserved.
  * Copyright (c) 2009      Sun Microsystems, Inc.  All rights reserved.
  * $COPYRIGHT$
  * 
@@ -51,6 +51,10 @@ BEGIN_C_DECLS
 
 static int cid_block_start = 28;
 
+static int ompi_comm_cid_checkforreuse ( int c_id_start_index, int block );
+static int ompi_comm_get_blocksize ( ompi_communicator_t* comm  );
+
+
 typedef int ompi_comm_cid_allredfct (int *inbuf, int* outbuf, 
                                      int count, struct ompi_op_t *op, 
                                      ompi_communicator_t *comm,
@@ -130,6 +134,7 @@ int ompi_comm_nextcid ( ompi_communicator_t* newcomm,
      * Determine which implementation of allreduce we have to use
      * for the current scenario 
      */
+
     switch (mode) 
     {
         case OMPI_COMM_CID_INTRA: 
@@ -149,11 +154,17 @@ int ompi_comm_nextcid ( ompi_communicator_t* newcomm,
             break;
     }
 
+    /*
+     * BWB: Always make the multi-threaded algorithm the selected
+     * algorithm.  The others don't reuse CIDs which leads to CID
+     * space exhaustion in real-world applications. Leave the other
+     * code, as Edgar plans on adding CID reuse in the not-to-distant
+     * future. */
     /**
      * In case multi-threading is enabled, we revert to the old algorithm
      * starting from cid_block_start
      */
-    if (MPI_THREAD_MULTIPLE == ompi_mpi_thread_provided) {
+    if (1 || MPI_THREAD_MULTIPLE == ompi_mpi_thread_provided) {
         int nextlocal_cid;
         int done=0;
         int response, glresponse=0;
@@ -244,7 +255,7 @@ int ompi_comm_nextcid ( ompi_communicator_t* newcomm,
       * In case the communication mode is INTRA_OOB or INTAR_BRIDGE, we use the 
       * highest-free algorithm
       */
-    if ( OMPI_COMM_CID_INTRA_OOB == mode || OMPI_COMM_CID_INTRA_BRIDGE == mode) {
+    if ( OMPI_COMM_CID_INTRA_OOB == mode || OMPI_COMM_CID_INTRA_BRIDGE == mode) {       
         (allredfnct)(&cid_block_start, &global_block_start, 1, 
                      MPI_MAX, comm, bridgecomm,
                      local_leader, remote_leader, send_first );
@@ -253,41 +264,59 @@ int ompi_comm_nextcid ( ompi_communicator_t* newcomm,
         cid_block_start = cid_block_start + 1;
     }
     else {
-        flag=false;
-        block = 0;
-        if( 0 == comm->c_contextid ) {
-            block = OMPI_COMM_BLOCK_WORLD;
-        }
-        else {
-            block = OMPI_COMM_BLOCK_OTHERS;
-        }
+    
+        block = ompi_comm_get_blocksize ( comm  ); 
 
-        while(!flag) {
-            /**
-             * If the communicator has IDs available then allocate one for the child
-             */
-            if(MPI_UNDEFINED != comm->c_id_available && 
-               MPI_UNDEFINED != comm->c_id_start_index &&  
-               block > comm->c_id_available - comm->c_id_start_index) {
-                nextcid = comm->c_id_available;
-                flag=opal_pointer_array_test_and_set_item (&ompi_mpi_communicators,
-                                                           nextcid, comm);
-            }
-            /**
-             * Otherwise the communicator needs to negotiate a new block of IDs
-             */
-            else {
-                (allredfnct)(&cid_block_start, &global_block_start, 1, 
-                             MPI_MAX, comm, bridgecomm,
-                             local_leader, remote_leader, send_first );
-                cid_block_start = global_block_start;
-                comm->c_id_available = cid_block_start;
-                comm->c_id_start_index = cid_block_start;
-                cid_block_start = cid_block_start + block;
-            }
-        }
-        
-        comm->c_id_available++;
+	/**
+	 * If the communicator has IDs available then allocate one for the child
+	 */
+	if ( MPI_UNDEFINED != comm->c_id_available && 
+	     MPI_UNDEFINED != comm->c_id_start_index &&  
+	     block > comm->c_id_available - comm->c_id_start_index) {
+	    nextcid = comm->c_id_available;
+	    flag=opal_pointer_array_test_and_set_item (&ompi_mpi_communicators,
+						       nextcid, comm);
+	}
+	/**
+	 * Otherwise the communicator needs to negotiate a new block of IDs
+	 */
+	else {
+	    int start[3], gstart[3];
+	    /* the next function either returns exactly the same start_id as 
+	       the communicator had, or the cid_block_start*/
+	    start[0] = ompi_comm_cid_checkforreuse ( comm->c_id_start_index, block );
+	    
+	    /* this is now a little tricky. By multiplying the start[0] values with -1
+	       and executing the MAX operation on those as well, we will be able to
+	       determine the minimum value across the provided input */ 
+	    start[1] = (-1) * start[0]; 
+	    start[2] = cid_block_start;
+	    
+	    (allredfnct)(start, gstart, 3, MPI_MAX, comm, bridgecomm,
+			 local_leader, remote_leader, send_first );
+	    
+	    /* revert the minimum value back to a positive number */
+	    gstart[1] = (-1) * gstart[1];
+	    
+	    if  ( gstart[0] == start[0] && 
+		  gstart[1] == start[0] && 
+		  gstart[0] != cid_block_start ) {
+		comm->c_id_available   = gstart[0];
+		comm->c_id_start_index = gstart[0];
+		nextcid                = comm->c_id_available;
+		/* note: cid_block_start not modified in this section */
+	    }
+	    else {
+		/* no, one process did not agree on the reuse of the block
+		   so we have to go with the higher number */
+		comm->c_id_available   = gstart[2];
+		comm->c_id_start_index = gstart[2];
+		cid_block_start        = gstart[2] + block;
+		nextcid                = comm->c_id_available;
+	    }
+	}
+	
+	comm->c_id_available++;
     }
     /* set the according values to the newcomm */
     newcomm->c_contextid = nextcid;
@@ -406,10 +435,56 @@ static uint32_t ompi_comm_lowest_cid (void)
  * comm.c is, that this file contains the allreduce implementations
  * which are required, and thus we avoid having duplicate code...
  */
-int ompi_comm_activate ( ompi_communicator_t** newcomm )
+int ompi_comm_activate ( ompi_communicator_t** newcomm, 
+                         ompi_communicator_t* comm,
+                         ompi_communicator_t* bridgecomm,
+                         void* local_leader,
+                         void* remote_leader,
+                         int mode,
+                         int send_first )
 {
     int ret = 0;
 
+    int ok=0, gok=0;
+    ompi_comm_cid_allredfct* allredfnct;
+
+    /* Step 1: the barrier, after which it is allowed to
+     * send messages over the new communicator
+     */
+    switch (mode)
+    {
+        case OMPI_COMM_CID_INTRA:
+            allredfnct=(ompi_comm_cid_allredfct*)ompi_comm_allreduce_intra;
+            break;
+        case OMPI_COMM_CID_INTER:
+            allredfnct=(ompi_comm_cid_allredfct*)ompi_comm_allreduce_inter;
+            break;
+        case OMPI_COMM_CID_INTRA_BRIDGE:
+            allredfnct=(ompi_comm_cid_allredfct*)ompi_comm_allreduce_intra_bridge;
+            break;
+        case OMPI_COMM_CID_INTRA_OOB:
+            allredfnct=(ompi_comm_cid_allredfct*)ompi_comm_allreduce_intra_oob;
+            break;
+        default:
+            return MPI_UNDEFINED;
+            break;
+    }
+
+    if (MPI_UNDEFINED != (*newcomm)->c_local_group->grp_my_rank) {
+
+	/* Initialize the PML stuff in the newcomm  */
+	if ( OMPI_SUCCESS != (ret = MCA_PML_CALL(add_comm(*newcomm))) ) {
+	    goto bail_on_error;
+	}
+	OMPI_COMM_SET_PML_ADDED(*newcomm);
+    }
+
+
+    (allredfnct)(&ok, &gok, 1, MPI_MIN, comm, bridgecomm,
+                 local_leader, remote_leader, send_first );
+
+
+
     /**
      * Check to see if this process is in the new communicator.
      *
@@ -437,16 +512,11 @@ int ompi_comm_activate ( ompi_communicator_t** newcomm )
     if (MPI_UNDEFINED == (*newcomm)->c_local_group->grp_my_rank) {
         return OMPI_SUCCESS;
     }
-    /* Initialize the PML stuff in the newcomm  */
-    if ( OMPI_SUCCESS != (ret = MCA_PML_CALL(add_comm(*newcomm))) ) {
-        goto bail_on_error;
-    }
-    OMPI_COMM_SET_PML_ADDED(*newcomm);
 
     /* Let the collectives components fight over who will do
        collective on this new comm.  */
     if (OMPI_SUCCESS != (ret = mca_coll_base_comm_select(*newcomm))) {
-        goto bail_on_error;
+	goto bail_on_error;
     }
     return OMPI_SUCCESS;
 
@@ -456,6 +526,80 @@ int ompi_comm_activate ( ompi_communicator_t** newcomm )
     return ret;
 }                         
 
+/**************************************************************************/
+/**************************************************************************/
+/**************************************************************************/
+/* check whether all communicators registered from c_id_start_index to 
+** c_id_start_index + block have been freed. For this, we rely on 
+** the communicators having been properly removed from the fortran array, 
+** i.e. the according request should return a NULL pointer. 
+*/
+static int ompi_comm_cid_checkforreuse ( int c_id_start_index, int block )
+{
+    int ret=cid_block_start;
+    int i, count=0;
+    ompi_communicator_t * tempcomm;
+
+    if ( MPI_UNDEFINED != c_id_start_index ) {
+	for ( i= c_id_start_index; i < c_id_start_index + block; i++ ) {
+	    tempcomm = (ompi_communicator_t *) opal_pointer_array_get_item ( &ompi_mpi_communicators, i );
+	    if ( NULL == tempcomm ) {
+		count++;
+	    }
+	}
+	
+	if ( count == block ) {
+	    ret = c_id_start_index;
+	}
+    }
+
+    return ret;
+}
+
+/* this function is called from the communicator destructor. It is a
+   generic interface which verifies whether the cid block assigned to 
+   that communicator can be reused. 
+   In the current implementation it checks whether all communicators 
+   have been freed. If that's the case *and* the current leading fron
+   of cid's (i.e. cid_block-start) is right after the block following
+   the block assigned to that communicator, we reset the leading block.
+*/
+void  ompi_comm_checkfor_blockreset ( ompi_communicator_t *comm )
+{
+    int block=ompi_comm_get_blocksize ( comm );
+    int next=0;
+
+    if (MPI_THREAD_MULTIPLE == ompi_mpi_thread_provided) {
+	return;
+    }
+
+    if ( MPI_UNDEFINED == comm->c_id_start_index ) {
+	return;
+    }
+
+    next = ompi_comm_cid_checkforreuse ( comm->c_id_start_index, block );
+    if ( next            == comm->c_id_start_index && 
+	 cid_block_start == (comm->c_id_start_index + block ) ) {
+ 	cid_block_start = comm->c_id_start_index;
+    }
+
+    return;
+}
+
+static int ompi_comm_get_blocksize ( ompi_communicator_t* comm  ) 
+{
+    int block=0;
+
+    if( 0 == comm->c_contextid ) {
+	block = OMPI_COMM_BLOCK_WORLD;
+    }
+    else {
+	block = OMPI_COMM_BLOCK_OTHERS;
+    }
+
+    return block;
+}
+
 /**************************************************************************/
 /**************************************************************************/
 /**************************************************************************/
