@@ -2,6 +2,7 @@
  * Copyright (c) 2007-2009 Cisco Systems, Inc.  All rights reserved.
  * Copyright (c) 2007-2008 Chelsio, Inc. All rights reserved.
  * Copyright (c) 2008      Mellanox Technologies. All rights reserved.
+ * Copyright (c) 2009      Sandia National Laboratories. All rights reserved.
  *
  * $COPYRIGHT$
  *
@@ -27,6 +28,7 @@
 #include <ctype.h>
 #include <dirent.h>
 #include <malloc.h>
+#include <stddef.h>
 
 #include "opal/util/argv.h"
 #include "opal/util/error.h"
@@ -119,6 +121,7 @@ typedef struct {
     mca_btl_openib_endpoint_t *endpoint;
     uint8_t qpnum;
     bool already_disconnected;
+    uint16_t route_retry_count;
     struct rdma_cm_id *id;
 } id_context_t;
 
@@ -150,15 +153,15 @@ static struct rdma_event_channel *event_channel = NULL;
 static int rdmacm_priority = 30;
 static uint16_t rdmacm_port = 0;
 static uint32_t rdmacm_addr = 0;
-static int rdmacm_resolve_timeout = 2000;
+static int rdmacm_resolve_timeout = 1000;
+static int rdmacm_resolve_max_retry_count = 20;
 static bool rdmacm_reject_causes_connect_error = false;
 static volatile int disconnect_callbacks = 0;
 static bool rdmacm_component_initialized = false;
 
 /* Calculate the *real* length of the message (not aligned/rounded
    up) */
-static modex_message_t dummy_message;
-static int message_len = ((char*) &(dummy_message.end)) - ((char*) &dummy_message);
+static int message_len = offsetof(modex_message_t, end);
 
 /* Rejection reasons */
 typedef enum {
@@ -173,6 +176,7 @@ static void id_context_constructor(id_context_t *context)
     context->contents = NULL;
     context->endpoint = NULL;
     context->qpnum = 255;
+    context->route_retry_count = 0;
 }
 
 static void id_context_destructor(id_context_t *context)
@@ -245,6 +249,17 @@ static void rdmacm_component_register(void)
                        "illegal timeout", true, value);
     }
 
+    mca_base_param_reg_int(&mca_btl_openib_component.super.btl_version,
+                           "connect_rdmacm_retry_count",
+                           "Maximum number of times rdmacm will retry route resolution",
+                           false, false, rdmacm_resolve_max_retry_count, &value);
+    if (value > 0) {
+        rdmacm_resolve_max_retry_count = value;
+    } else {
+        orte_show_help("help-mpi-btl-openib-cpc-rdmacm.txt",
+                       "illegal retry count", true, value);
+    }
+
     mca_base_param_reg_int(&mca_btl_openib_component.super.btl_version,
                            "connect_rdmacm_reject_causes_connect_error",
                            "The drivers for some devices are buggy such that an RDMA REJECT action may result in a CONNECT_ERROR event instead of a REJECTED event.  Setting this MCA parameter to true tells Open MPI to treat CONNECT_ERROR events on connections where a REJECT is expected as a REJECT (default: false)",
@@ -427,6 +442,7 @@ static int rdmacm_setup_qp(rdmacm_contents_t *contents,
                        "inline truncated", true,
                        orte_process_info.nodename,
                        ibv_get_device_name(contents->openib_btl->device->ib_dev),
+                       contents->openib_btl->port_num,
                        req_inline, attr.cap.max_inline_data);
     } else {
         endpoint->qps[qpnum].ib_inline_max = req_inline;
@@ -540,14 +556,6 @@ static int rdmacm_client_connect_one(rdmacm_contents_t *contents,
     free(a);
     free(b);
 #endif
-    rc = rdma_resolve_addr(context->id,
-                           (struct sockaddr *) &src_in,
-                           (struct sockaddr *) &dest_in,
-                           rdmacm_resolve_timeout);
-    if (0 != rc) {
-        BTL_ERROR(("Failed to resolve the remote address with %d", rc));
-        goto out1;
-    }
 
     /* This is odd an worth explaining: when we place the context on
        the ids list, we need to add an extra RETAIN to the context.
@@ -586,6 +594,16 @@ static int rdmacm_client_connect_one(rdmacm_contents_t *contents,
      */
     OBJ_RETAIN(context);
     opal_list_append(&(contents->ids), &(context->super));
+
+    rc = rdma_resolve_addr(context->id,
+                           (struct sockaddr *) &src_in,
+                           (struct sockaddr *) &dest_in,
+                           rdmacm_resolve_timeout);
+    if (0 != rc) {
+        BTL_ERROR(("Failed to resolve the remote address with %d", rc));
+        goto out1;
+    }
+
     return OMPI_SUCCESS;
 
 out1:
@@ -959,7 +977,7 @@ static void *call_disconnect_callback(void *v)
     OBJ_RELEASE(context);
 
     /* Tell the main thread that we're done */
-    ++disconnect_callbacks;
+    opal_atomic_add(&disconnect_callbacks, 1);
     OPAL_OUTPUT((-1, "SERVICE Service thread disconnect on ID %p done; count=%d",
                  (void*) tmp, disconnect_callbacks));
     return NULL;
@@ -1002,9 +1020,8 @@ static int rdmacm_endpoint_finalize(struct mca_btl_base_endpoint_t *endpoint)
         rdmacm_contents_t *contents = (rdmacm_contents_t *) item;
 
         if (endpoint == contents->endpoint) {
-            for (item2 = opal_list_remove_first(&(contents->ids));
-                 NULL != item2;
-                 item2 = opal_list_remove_first(&(contents->ids))) {
+            while (NULL != 
+                   (item2 = opal_list_remove_first(&(contents->ids)))) {
                 /* Fun race condition: we cannot call
                    rdma_disconnect() here in the main thread, because
                    if we do, there is a nonzero chance that the
@@ -1021,7 +1038,9 @@ static int rdmacm_endpoint_finalize(struct mca_btl_base_endpoint_t *endpoint)
                 ompi_btl_openib_fd_run_in_service(call_disconnect_callback,
                                                   item2);
             }
-            opal_list_remove_item(&client_list, item);
+	    /* remove_item returns the item before the item removed,
+	       meaning that the for list is still safe */
+            item = opal_list_remove_item(&client_list, item);
             contents->on_client_list = false;
             break;
         }
@@ -1033,6 +1052,7 @@ static int rdmacm_endpoint_finalize(struct mca_btl_base_endpoint_t *endpoint)
 
     /* Now wait for all the disconnect callbacks to occur */
     while (num_to_wait_for != disconnect_callbacks) {
+        ompi_btl_openib_fd_main_thread_drain();
         sched_yield();
     }
 
@@ -1137,10 +1157,16 @@ static int rdmacm_disconnected(id_context_t *context)
  */
 static int rdmacm_destroy_dummy_qp(id_context_t *context)
 {
-    if (NULL != context->id->qp) {
-        ibv_destroy_qp(context->id->qp);
-        context->id->qp = NULL;
+    /* We need to check id pointer because of retransmitions.
+       Maybe the reject was already done. */
+
+    if (NULL != context->id) {
+	    if (NULL != context->id->qp) {
+           ibv_destroy_qp(context->id->qp);
+           context->id->qp = NULL;
+        }
     }
+
     if (NULL != context->contents->dummy_cq) {
         ibv_destroy_cq(context->contents->dummy_cq);
     }
@@ -1539,12 +1565,26 @@ static int event_handler(struct rdma_cm_event *event)
     case RDMA_CM_EVENT_UNREACHABLE:
     case RDMA_CM_EVENT_CONNECT_RESPONSE:
     case RDMA_CM_EVENT_ADDR_ERROR:
-    case RDMA_CM_EVENT_ROUTE_ERROR:
     case RDMA_CM_EVENT_DEVICE_REMOVAL:
         ompi_btl_openib_fd_run_in_main(show_help_rdmacm_event_error, event);
         rc = OMPI_ERROR;
         break;
 
+    case RDMA_CM_EVENT_ROUTE_ERROR:
+        /* Route lookup does not necessarily handle retries, and there
+           appear to be cases where the subnet manager node can no
+           longer handle incoming requests.  The rdma connection
+           manager and lower level code doesn't handle retries, so we
+           have to. */
+	if (context->route_retry_count < rdmacm_resolve_max_retry_count) {
+		context->route_retry_count++;
+		rc = resolve_route(context);
+		break;
+	}
+        ompi_btl_openib_fd_run_in_main(show_help_rdmacm_event_error, event);
+        rc = OMPI_ERROR;
+        break;
+
     default:
         /* Unknown error */
         BTL_ERROR(("Unknown RDMA CM error event_handler: %s, status = %d",
@@ -1752,7 +1792,9 @@ static int rdmacm_component_query(mca_btl_openib_module_t *openib_btl, ompi_btl_
 
     /* RDMACM is not supported if we have any XRC QPs */
     if (mca_btl_openib_component.num_xrc_qps > 0) {
-        BTL_VERBOSE(("rdmacm CPC not supported with XRC receive queues, please try xoob CPC; skipped"));
+        BTL_VERBOSE(("rdmacm CPC not supported with XRC receive queues, please try xoob CPC; skipped on %s:%d",
+                     ibv_get_device_name(openib_btl->device->ib_dev),
+                     openib_btl->port_num));
         rc = OMPI_ERR_NOT_SUPPORTED;
         goto out;
     }
@@ -1853,8 +1895,9 @@ static int rdmacm_component_query(mca_btl_openib_module_t *openib_btl, ompi_btl_
     opal_list_append(&server_listener_list, &(server->super));
 
     opal_output_verbose(5, mca_btl_base_output,
-                        "openib BTL: rdmacm CPC available for use on %s",
-                        ibv_get_device_name(openib_btl->device->ib_dev));
+                        "openib BTL: rdmacm CPC available for use on %s:%d",
+                        ibv_get_device_name(openib_btl->device->ib_dev),
+                        openib_btl->port_num);
     return OMPI_SUCCESS;
 
 out5:
@@ -1869,12 +1912,14 @@ out1:
 out:
     if (OMPI_ERR_NOT_SUPPORTED == rc) {
         opal_output_verbose(5, mca_btl_base_output,
-                            "openib BTL: rdmacm CPC unavailable for use on %s; skipped",
-                            ibv_get_device_name(openib_btl->device->ib_dev));
+                            "openib BTL: rdmacm CPC unavailable for use on %s:%d; skipped",
+                            ibv_get_device_name(openib_btl->device->ib_dev),
+                            openib_btl->port_num);
     } else {
         opal_output_verbose(5, mca_btl_base_output,
-                            "openib BTL: rmacm CPC unavailable for use on %s; fatal error %d (%s)",
-                            ibv_get_device_name(openib_btl->device->ib_dev), rc,
+                            "openib BTL: rmacm CPC unavailable for use on %s:%d; fatal error %d (%s)",
+                            ibv_get_device_name(openib_btl->device->ib_dev), 
+                            openib_btl->port_num, rc,
                             opal_strerror(rc));
     }
     return rc;
