@@ -411,6 +411,7 @@ static int xoob_send_qp_create (mca_btl_base_endpoint_t* endpoint)
         orte_show_help("help-mpi-btl-openib-cpc-base.txt",
                        "inline truncated", orte_process_info.nodename,
                        ibv_get_device_name(openib_btl->device->ib_dev),
+                       openib_btl->port_num,
                        req_inline, qp_init_attr.cap.max_inline_data);
     } else {
         endpoint->qps[0].ib_inline_max = req_inline;
@@ -811,6 +812,7 @@ static void xoob_rml_recv_cb(int status, orte_process_name_t* process_name,
             if (OMPI_SUCCESS != mca_btl_openib_endpoint_post_recvs(ib_endpoint)) {
                 BTL_ERROR(("Failed to post on XRC SRQs"));
                 mca_btl_openib_endpoint_invoke_error(NULL);
+                OPAL_THREAD_UNLOCK(&ib_endpoint->endpoint_lock);
                 return;
             }
             /* we should create qp and send the info + srq to requestor */
@@ -818,6 +820,7 @@ static void xoob_rml_recv_cb(int status, orte_process_name_t* process_name,
             if (OMPI_SUCCESS != rc) {
                 BTL_ERROR(("error in endpoint reply start connect"));
                 mca_btl_openib_endpoint_invoke_error(NULL);
+                OPAL_THREAD_UNLOCK(&ib_endpoint->endpoint_lock);
                 return;
             }
             /* enable pooling for this btl */
@@ -848,6 +851,7 @@ static void xoob_rml_recv_cb(int status, orte_process_name_t* process_name,
                 if (OMPI_SUCCESS != rc) {
                     BTL_ERROR(("error in endpoint reply start connect"));
                     mca_btl_openib_endpoint_invoke_error(ib_endpoint);
+                    OPAL_THREAD_UNLOCK(&ib_endpoint->endpoint_lock);
                     return;
                 }
                 OPAL_THREAD_UNLOCK(&ib_endpoint->endpoint_lock);
@@ -858,6 +862,7 @@ static void xoob_rml_recv_cb(int status, orte_process_name_t* process_name,
                 if (OMPI_SUCCESS != rc) {
                     BTL_ERROR(("error in endpoint reply start connect"));
                     mca_btl_openib_endpoint_invoke_error(ib_endpoint);
+                    OPAL_THREAD_UNLOCK(&ib_endpoint->endpoint_lock);
                     return;
                 }
                 OPAL_THREAD_UNLOCK(&ib_endpoint->endpoint_lock);
@@ -890,10 +895,11 @@ static void xoob_rml_recv_cb(int status, orte_process_name_t* process_name,
             if (OMPI_SUCCESS != xoob_send_qp_connect(ib_endpoint, &rem_info)) {
                 BTL_ERROR(("Failed to connect  endpoint\n"));
                 mca_btl_openib_endpoint_invoke_error(NULL);
+                OPAL_THREAD_UNLOCK(&ib_endpoint->endpoint_lock);
                 return;
             }
             mca_btl_openib_endpoint_cpc_complete(ib_endpoint);
-            OPAL_THREAD_UNLOCK(&ib_endpoint->endpoint_lock);
+            /* cpc complete unlock the endpoint */
             break;
         case ENDPOINT_XOOB_CONNECT_XRC_RESPONSE:
             BTL_VERBOSE(("Received ENDPOINT_XOOB_CONNECT_XRC_RESPONSE: lid %d, sid %d\n",
@@ -912,7 +918,7 @@ static void xoob_rml_recv_cb(int status, orte_process_name_t* process_name,
             /* we got srq numbers on our request */
             XOOB_SET_REMOTE_INFO(ib_endpoint->rem_info, rem_info);
             mca_btl_openib_endpoint_cpc_complete(ib_endpoint);
-            OPAL_THREAD_UNLOCK(&ib_endpoint->endpoint_lock);
+            /* cpc complete unlock the endpoint */
             break;
         case ENDPOINT_XOOB_CONNECT_XRC_NR_RESPONSE:
             /* The XRC recv site already was destroyed so we need
@@ -950,8 +956,9 @@ static int xoob_component_query(mca_btl_openib_module_t *openib_btl,
 
     if (mca_btl_openib_component.num_xrc_qps <= 0) {
         opal_output_verbose(5, mca_btl_base_output,
-                            "openib BTL: xoob CPC only supported with XRC receive queues; skipped on device %s",
-                            ibv_get_device_name(openib_btl->device->ib_dev));
+                            "openib BTL: xoob CPC only supported with XRC receive queues; skipped on %s:%d",
+                            ibv_get_device_name(openib_btl->device->ib_dev),
+                            openib_btl->port_num);
         return OMPI_ERR_NOT_SUPPORTED;
     }
 
@@ -992,8 +999,9 @@ static int xoob_component_query(mca_btl_openib_module_t *openib_btl,
     (*cpc)->cbm_uses_cts = false;
 
     opal_output_verbose(5, mca_btl_base_output,
-                        "openib BTL: xoob CPC available for use on %s",
-                        ibv_get_device_name(openib_btl->device->ib_dev));
+                        "openib BTL: xoob CPC available for use on %s:%d",
+                        ibv_get_device_name(openib_btl->device->ib_dev),
+                        openib_btl->port_num);
     return OMPI_SUCCESS;
 }
 
