@@ -2,7 +2,7 @@
  * Copyright (c) 2004-2005 The Trustees of Indiana University and Indiana
  *                         University Research and Technology
  *                         Corporation.  All rights reserved.
- * Copyright (c) 2004-2005 The University of Tennessee and The University
+ * Copyright (c) 2004-2006 The University of Tennessee and The University
  *                         of Tennessee Research Foundation.  All rights
  *                         reserved.
  * Copyright (c) 2004-2005 High Performance Computing Center Stuttgart, 
@@ -20,8 +20,8 @@
 #define MCA_BTL_TCP_PROC_H
 
 #include "opal/class/opal_object.h"
-#include "orte/mca/ns/ns.h"
 #include "ompi/proc/proc.h"
+#include "orte/types.h"
 #include "btl_tcp.h"
 #include "btl_tcp_addr.h"
 #include "btl_tcp_endpoint.h"
@@ -29,7 +29,6 @@
 #if defined(c_plusplus) || defined(__cplusplus)
 extern "C" {
 #endif
-OBJ_CLASS_DECLARATION(mca_btl_tcp_proc_t);
 
 /**
  * Represents the state of a remote process and the set of addresses
@@ -63,12 +62,53 @@ struct mca_btl_tcp_proc_t {
     /**< lock to protect against concurrent access to proc state */
 };
 typedef struct mca_btl_tcp_proc_t mca_btl_tcp_proc_t;
+OBJ_CLASS_DECLARATION(mca_btl_tcp_proc_t);
+
+/*	the highest possible interface kernel index we can handle */
+#define MAX_KERNEL_INTERFACE_INDEX 65536
+
+/*	the maximum number of kernel interfaces we can handle */
+#define MAX_KERNEL_INTERFACES 8
+
+/*
+ * FIXME: this should probably be part of an ompi list, so we need the
+ * appropriate definitions
+ */
+
+struct mca_btl_tcp_interface_t {
+	struct sockaddr_storage* ipv4_address;
+	struct sockaddr_storage* ipv6_address;
+	mca_btl_tcp_addr_t* ipv4_endpoint_addr;
+	mca_btl_tcp_addr_t* ipv6_endpoint_addr;
+	uint32_t ipv4_netmask;
+	uint32_t ipv6_netmask;
+	int kernel_index;
+	int peer_interface;
+	int index;
+	int inuse;
+};
+
+typedef struct mca_btl_tcp_interface_t mca_btl_tcp_interface_t;
+
+/*
+ * describes the quality of a possible connection between a local and
+ * a remote network interface
+ */
+enum mca_btl_tcp_connection_quality { 
+	CQ_NO_CONNECTION,
+	CQ_PRIVATE_DIFFERENT_NETWORK,
+	CQ_PRIVATE_SAME_NETWORK,
+	CQ_PUBLIC_DIFFERENT_NETWORK,
+	CQ_PUBLIC_SAME_NETWORK
+};
+
 
 mca_btl_tcp_proc_t* mca_btl_tcp_proc_create(ompi_proc_t* ompi_proc);
 mca_btl_tcp_proc_t* mca_btl_tcp_proc_lookup(const orte_process_name_t* name);
 int  mca_btl_tcp_proc_insert(mca_btl_tcp_proc_t*, mca_btl_base_endpoint_t*);
 int  mca_btl_tcp_proc_remove(mca_btl_tcp_proc_t*, mca_btl_base_endpoint_t*);
-bool mca_btl_tcp_proc_accept(mca_btl_tcp_proc_t*, struct sockaddr_in*, int);
+bool mca_btl_tcp_proc_accept(mca_btl_tcp_proc_t*, struct sockaddr*, int);
+bool mca_btl_tcp_proc_tosocks(mca_btl_tcp_addr_t*, struct sockaddr_storage*);
 
 /**
  * Inlined function to return local TCP proc instance.
