@@ -10,7 +10,8 @@
  * Copyright (c) 2004-2005 The Regents of the University of California.
  *                         All rights reserved.
  * Copyright (c) 2007      Cisco, Inc.  All rights reserved.
- * Copyright (c) 2006-2008 University of Houston.  All rights reserved.
+ * Copyright (c) 2006-2009 University of Houston.  All rights reserved.
+ * Copyright (c) 2009      Sun Microsystems, Inc.  All rights reserved.
  * $COPYRIGHT$
  *
  * Additional copyrights may follow
@@ -42,6 +43,7 @@
 #include "orte/runtime/orte_wait.h"
 
 #include "ompi/communicator/communicator.h"
+#include "ompi/group/group.h"
 #include "ompi/proc/proc.h"
 #include "ompi/mca/pml/pml.h"
 #include "ompi/info/info.h"
@@ -426,7 +428,13 @@ static int connect_accept ( ompi_communicator_t *comm, int root,
     }
 
     /* activate comm and init coll-component */
-    rc = ompi_comm_activate ( &newcomp );             /* new communicator */
+    rc = ompi_comm_activate ( &newcomp,               /* new communicator */
+                             comm,                    /* old communicator */
+                             NULL,                    /* bridge comm */
+                             &root,                   /* local leader */
+                             &carport,                /* remote leader */
+                             OMPI_COMM_CID_INTRA_OOB, /* mode */
+                             send_first );            /* send or recv first */
     if ( OMPI_SUCCESS != rc ) {
         goto exit;
     }
@@ -951,11 +959,11 @@ static int dyn_init(void)
      * objects
      */
     
-    oldcomm = &ompi_mpi_comm_null;
+    oldcomm = &ompi_mpi_comm_null.comm;
     OBJ_RELEASE(oldcomm);
-    group = &ompi_mpi_group_null;
+    group = &ompi_mpi_group_null.group;
     OBJ_RELEASE(group);
-    errhandler = &ompi_mpi_errors_are_fatal;
+    errhandler = &ompi_mpi_errors_are_fatal.eh;
     OBJ_RELEASE(errhandler);
     
     /* Set name for debugging purposes */
