@@ -29,6 +29,7 @@
 #include "orte/mca/errmgr/errmgr.h"
 #include "orte/util/name_fns.h"
 #include "orte/runtime/orte_globals.h"
+#include "orte/mca/notifier/notifier.h"
 
 #include "ompi/mca/pml/pml.h"
 #include "ompi/mca/bml/bml.h" 
@@ -44,6 +45,28 @@
 #include "pml_csum_sendreq.h"
 #include "pml_csum_rdmafrag.h"
 
+/**
+ * Dump data elements that caused a checksum violation
+ */
+static void dump_csum_error_data(mca_btl_base_segment_t* segments, size_t num_segments)
+{
+    size_t i, j;
+    uint8_t *data;
+    
+    printf("CHECKSUM ERROR DATA\n");
+    for (i = 0; i < num_segments; ++i) {
+        printf("Segment %lu", i);
+        data = (uint8_t*)segments[i].seg_addr.pval;
+        for (j=0; j < segments[i].seg_len; j++) {
+            if (0 == (j % 40)) {
+                printf("\n");
+            }
+            printf("%02x ", data[j]);
+        };
+    }
+    printf("\nEND CHECKSUM ERROR DATA\n\n");
+}
+
 void mca_pml_csum_recv_request_process_pending(void)
 {
     mca_pml_csum_recv_request_t* recvreq;
@@ -473,6 +496,11 @@ void mca_pml_csum_recv_request_progress_frag( mca_pml_csum_recv_request_t* recvr
         if(csum != hdr->hdr_frag.hdr_csum) {
             opal_output(0, "%s:%s:%d: Invalid \'frag data\' - received csum:0x%x  != computed csum:0x%x\n",
                         ORTE_NAME_PRINT(ORTE_PROC_MY_NAME), __FILE__, __LINE__, hdr->hdr_frag.hdr_csum, csum);
+            orte_notifier.log(ORTE_NOTIFIER_INFRA,
+                              "Checksum data violation: job %s file %s line %d",
+                              (NULL == orte_job_ident) ? "UNKNOWN" : orte_job_ident,
+                              __FILE__, __LINE__);
+            dump_csum_error_data(segments, num_segments);
             orte_errmgr.abort(-1,NULL);
         }
     }
@@ -614,6 +642,11 @@ void mca_pml_csum_recv_request_progress_rndv( mca_pml_csum_recv_request_t* recvr
         if (csum != hdr->hdr_match.hdr_csum) {
             opal_output(0, "%s:%s:%d: Invalid \'rndv data\' - received csum:0x%x  != computed csum:0x%x\n",
                         ORTE_NAME_PRINT(ORTE_PROC_MY_NAME), __FILE__, __LINE__, hdr->hdr_match.hdr_csum, csum);
+            orte_notifier.log(ORTE_NOTIFIER_INFRA,
+                              "Checksum data violation: job %s file %s line %d",
+                              (NULL == orte_job_ident) ? "UNKNOWN" : orte_job_ident,
+                              __FILE__, __LINE__);
+            dump_csum_error_data(segments, num_segments);
             orte_errmgr.abort(-1,NULL);
         }
     }
@@ -671,6 +704,11 @@ void mca_pml_csum_recv_request_progress_match( mca_pml_csum_recv_request_t* recv
         if (csum != hdr->hdr_match.hdr_csum) {
             opal_output(0, "%s:%s:%d: Invalid \'match data\' - received csum:0x%x  != computed csum:0x%x\n",
                         ORTE_NAME_PRINT(ORTE_PROC_MY_NAME), __FILE__, __LINE__, hdr->hdr_match.hdr_csum, csum);
+            orte_notifier.log(ORTE_NOTIFIER_INFRA,
+                              "Checksum data violation: job %s file %s line %d",
+                              (NULL == orte_job_ident) ? "UNKNOWN" : orte_job_ident,
+                              __FILE__, __LINE__);
+            dump_csum_error_data(segments, num_segments);
             orte_errmgr.abort(-1,NULL);
         }
     }
