@@ -2,13 +2,14 @@
  * Copyright (c) 2004-2005 The Trustees of Indiana University and Indiana
  *                         University Research and Technology
  *                         Corporation.  All rights reserved.
- * Copyright (c) 2004-2005 The University of Tennessee and The University
+ * Copyright (c) 2004-2006 The University of Tennessee and The University
  *                         of Tennessee Research Foundation.  All rights
  *                         reserved.
  * Copyright (c) 2004-2005 High Performance Computing Center Stuttgart, 
  *                         University of Stuttgart.  All rights reserved.
  * Copyright (c) 2004-2005 The Regents of the University of California.
  *                         All rights reserved.
+ * Copyright (c) 2007      Cisco, Inc.  All rights reserved.
  * $COPYRIGHT$
  * 
  * Additional copyrights may follow
@@ -23,6 +24,7 @@
 #include "opal_config.h"
 
 #include "opal/util/printf.h"
+#include "opal/util/output.h"
 
 #include <errno.h>
 #include <stdio.h>
@@ -38,7 +40,7 @@
  */
 static int guess_strlen(const char *fmt, va_list ap)
 {
-    char *sarg;
+    char *sarg, carg;
     double darg;
     float farg;
     size_t i;
@@ -49,22 +51,30 @@ static int guess_strlen(const char *fmt, va_list ap)
     /* Start off with a fudge factor of 128 to handle the % escapes that
        we aren't calculating here */
 
-    len = strlen(fmt) + 128;
+    len = (int)strlen(fmt) + 128;
     for (i = 0; i < strlen(fmt); ++i) {
         if ('%' == fmt[i] && i + 1 < strlen(fmt)
             && '%' != fmt[i + 1]) {
             ++i;
             switch (fmt[i]) {
+            case 'c':
+                carg = va_arg(ap, int);
+                len += 1;  /* let's suppose it's a printable char */
+                break;
             case 's':
                 sarg = va_arg(ap, char *);
 
                 /* If there's an arg, get the strlen, otherwise we'll
                  * use (null) */
 
-                if (NULL != sarg)
-                    len += strlen(sarg);
-                else
+                if (NULL != sarg) {
+                    len += (int)strlen(sarg);
+                } else {
+#if OMPI_ENABLE_DEBUG
+                    opal_output(0, "OPAL DEBUG WARNING: Got a NULL argument to opal_vasprintf!\n");
+#endif
                     len += 5;
+                }
                 break;
 
             case 'd':
@@ -91,7 +101,7 @@ static int guess_strlen(const char *fmt, va_list ap)
                 break;
 
             case 'f':
-                farg = va_arg(ap, int);
+                farg = (float)va_arg(ap, int);
                 /* Alloc for minus sign */
                 if (farg < 0) {
                     ++len;
@@ -197,11 +207,11 @@ int opal_vasprintf(char **ptr, const char *fmt, va_list ap)
        it twice is a bad idea.  So make a copy for the second
        use.  Copy order taken from Autoconf docs. */
 #if OMPI_HAVE_VA_COPY
-  va_copy(ap2, ap);
+    va_copy(ap2, ap);
 #elif OMPI_HAVE_UNDERSCORE_VA_COPY
-  __va_copy(ap2, ap);
+    __va_copy(ap2, ap);
 #else
-  memcpy (&ap2, &ap, sizeof(va_list));
+    memcpy (&ap2, &ap, sizeof(va_list));
 #endif
 
     /* guess the size */
@@ -211,12 +221,15 @@ int opal_vasprintf(char **ptr, const char *fmt, va_list ap)
     *ptr = (char *) malloc((size_t) length + 1);
     if (NULL == *ptr) {
         errno = ENOMEM;
+        va_end(ap2);
         return -1;
     }
 
     /* fill the buffer */
     length = vsprintf(*ptr, fmt, ap2);
+#if OMPI_HAVE_VA_COPY || OMPI_HAVE_UNDERSCORE_VA_COPY
     va_end(ap2);
+#endif  /* OMPI_HAVE_VA_COPY || OMPI_HAVE_UNDERSCORE_VA_COPY */
 
     /* realloc */
     *ptr = (char*) realloc(*ptr, (size_t) length + 1);
@@ -262,6 +275,9 @@ int opal_vsnprintf(char *str, size_t size, const char *fmt, va_list ap)
         }
     }
 
+    /* free allocated buffer */
+    free(buf);
+
     return length;
 }
 
