@@ -54,14 +54,11 @@
 #include "orte/mca/rml/rml.h"
 #include "orte/util/name_fns.h"
 #include "orte/util/show_help.h"
+#include "orte/util/show_help.h"
 #include "orte/runtime/orte_globals.h"
 
 #include "orte/mca/oob/tcp/oob_tcp.h"
 
-#if defined(__WINDOWS__)
-static opal_mutex_t windows_callback;
-#endif  /* defined(__WINDOWS__) */
-
 /*
  * Data structure for accepting connections.
  */
@@ -151,39 +148,6 @@ mca_oob_t mca_oob_tcp = {
     mca_oob_tcp_ft_event
 };
 
-#if defined(__WINDOWS__)
-static int oob_tcp_windows_progress_callback( void )
-{
-    opal_list_item_t* item;
-    mca_oob_tcp_msg_t* msg;
-	int event_count = 0;
-
-	/* Only one thread at the time is allowed to execute callbacks */
-	if( !opal_mutex_trylock(&windows_callback) )
-		return 0;
-
-    OPAL_THREAD_LOCK(&mca_oob_tcp_component.tcp_lock);
-    while(NULL != 
-         (item = opal_list_remove_first(&mca_oob_tcp_component.tcp_msg_completed))) {
-        msg = (mca_oob_tcp_msg_t*)item;
-        OPAL_THREAD_UNLOCK(&mca_oob_tcp_component.tcp_lock);
-        msg->msg_cbfunc( msg->msg_rc, 
-                         &msg->msg_peer, 
-                         msg->msg_uiov, 
-                         msg->msg_ucnt, 
-                         msg->msg_hdr.msg_tag, 
-                         msg->msg_cbdata);
-		event_count++;
-        OPAL_THREAD_LOCK(&mca_oob_tcp_component.tcp_lock);
-        MCA_OOB_TCP_MSG_RETURN(msg);
-    }
-    OPAL_THREAD_UNLOCK(&mca_oob_tcp_component.tcp_lock);
-
-	opal_mutex_unlock(&windows_callback);
-
-    return event_count;
-}
-#endif  /* defined(__WINDOWS__) */
 
 /*
  * Initialize global variables used w/in this module.
@@ -382,13 +346,6 @@ int mca_oob_tcp_component_open(void)
     mca_oob_tcp_component.tcp_listen_sd = -1;
     mca_oob_tcp_component.tcp_match_count = 0;
 
-#if defined(__WINDOWS__)
-    /* Register the libevent callback which will trigger the OOB
-     * completion callbacks. */
-    OBJ_CONSTRUCT(&windows_callback, opal_mutex_t);
-    opal_progress_register(oob_tcp_windows_progress_callback);
-#endif  /* defined(__WINDOWS__) */
-
     return ORTE_SUCCESS;
 }
 
@@ -401,8 +358,6 @@ int mca_oob_tcp_component_close(void)
     opal_list_item_t *item;
 
 #if defined(__WINDOWS__)
-    opal_progress_unregister(oob_tcp_windows_progress_callback);
-    OBJ_DESTRUCT( &windows_callback );
     WSACleanup();
 #endif  /* defined(__WINDOWS__) */
 
@@ -478,12 +433,24 @@ static void mca_oob_tcp_accept(int incoming_sd)
 
         sd = accept(incoming_sd, (struct sockaddr*)&addr, &addrlen);
         if(sd < 0) {
-            if(opal_socket_errno == EINTR) {
+            if(EINTR == opal_socket_errno) {
                 continue;
             }
             if(opal_socket_errno != EAGAIN && opal_socket_errno != EWOULDBLOCK) {
-                opal_output(0, "mca_oob_tcp_accept: accept() failed: %s (%d).", 
-                            strerror(opal_socket_errno), opal_socket_errno);
+                if(EMFILE == opal_socket_errno) {
+                    /*
+                     * Close incoming_sd so that orte_show_help will have a file
+                     * descriptor with which to open the help file.  We will be
+                     * exiting anyway, so we don't need to keep it open.
+                     */
+                    CLOSE_THE_SOCKET(incoming_sd);
+                    ORTE_ERROR_LOG(ORTE_ERR_SYS_LIMITS_SOCKETS);
+                    orte_show_help("help-orterun.txt", "orterun:sys-limit-sockets", true);
+                } else {
+                    opal_output(0, "mca_oob_tcp_accept: accept() failed: %s (%d).", 
+                                strerror(opal_socket_errno), opal_socket_errno);
+                }
+                orte_errmgr.abort(ORTE_ERROR_DEFAULT_EXIT_CODE, NULL);
             }
             return;
         }
@@ -551,7 +518,7 @@ mca_oob_tcp_create_listen(int *target_sd, unsigned short *target_port, uint16_t
         if (AF_INET6 == af_family) {
             int flg = 0;
             if (setsockopt (*target_sd, IPPROTO_IPV6, IPV6_V6ONLY,
-                            &flg, sizeof (flg)) < 0) {
+                            (char *) &flg, sizeof (flg)) < 0) {
                 opal_output(0,
                             "mca_oob_tcp_create_listen: unable to disable v4-mapped addresses\n");
             }
@@ -569,7 +536,7 @@ mca_oob_tcp_create_listen(int *target_sd, unsigned short *target_port, uint16_t
 
     /* Disable reusing ports */
     flags = 0;
-    if (setsockopt (*target_sd, SOL_SOCKET, SO_REUSEADDR, (void*)&flags, sizeof(flags)) < 0) {
+    if (setsockopt (*target_sd, SOL_SOCKET, SO_REUSEADDR, (const char *) &flags, sizeof(flags)) < 0) {
         opal_output(0, "mca_oob_tcp_create_listen: unable to unset the "
                     "SO_REUSEADDR option (%s:%d)\n",
                     strerror(opal_socket_errno), opal_socket_errno);
@@ -764,9 +731,14 @@ mca_oob_tcp_listen_thread(opal_object_t *obj)
 
                     if (opal_socket_errno != EAGAIN || 
                         opal_socket_errno != EWOULDBLOCK) {
-                        opal_output(0, "mca_oob_tcp_accept: accept() failed: %s (%d).",
-                                    strerror(opal_socket_errno), opal_socket_errno);
                         CLOSE_THE_SOCKET(pending_connection->fd);
+                        if(EMFILE == opal_socket_errno) {
+                            ORTE_ERROR_LOG(ORTE_ERR_SYS_LIMITS_SOCKETS);
+                            orte_show_help("help-orterun.txt", "orterun:sys-limit-sockets", true);
+                        } else {
+                            opal_output(0, "mca_oob_tcp_accept: accept() failed: %s (%d).",
+                                        strerror(opal_socket_errno), opal_socket_errno);
+                        }
                         goto done;
                     }
 
